from email.mime import base
import glob
import os
import subprocess
import platform
import time

script_dir = os.path.dirname(os.path.abspath(__file__))
print(script_dir)
print(os.getcwd())

viewer = os.getenv("EMBREE_VIEWER")
verbose = True

if (viewer == None):
    #print("Please set environment variable EMBREE_VIEWER before running this script.")
    viewer = os.path.sep.join([script_dir, "..", "..", "build", "embree_viewer"])
    c = input("EMBREE_VIEWER environment variable not set. Default is set to <embree_root>/build/viewer. Do you want to continue? Y/n")
    if c == "n":
        exit()

# permutations to be tested
shaders =  ["eyelight", "uv", "Ng", "primID", "occlusion"]
instancing = ["none", "geometry", "group", "multi_level" "flattened"]

t = time.time()
for fullpath in glob.iglob(script_dir + os.path.sep + "*.xml"):
    (path, filename) = os.path.split(fullpath)
    (basename, ext) = os.path.splitext(filename)

    for shader in shaders:
        command = f"{viewer} -i {fullpath} -shader {shader} --time 0.5"
        if not os.path.exists(path + os.path.sep + "reference"):
            os.mkdir(path + os.path.sep + "reference")

        imagepath = path + os.path.sep + "reference" + os.path.sep + "prim_" + basename +  "_" + shader + ".exr"
        if not os.path.exists(imagepath):
            if verbose:
                print("Rendering " + filename + " " + shader)
                print(command)
            os.system(command + " -o " + imagepath)

print (time.time() - t)


#!/usr/bin/env python3

## Copyright 2009-2021 Intel Corporation
## SPDX-License-Identifier: Apache-2.0

import sys
import json
import subprocess 

def include(key, val):
    if key == 'CodeBlock':
        [[id, classes, keyvals], code] = val
        for kv in keyvals:
            if kv[0] == 'include':
                src0 = subprocess.check_output(['pandoc', '-s', kv[1], '-t', 'json'])
                j = json.loads(src0)
                if type(j) is list: return recurse(j[1])
                else              : return recurse(j['blocks'])
            elif kv[0] == 'image':
                return {"t":"Para","c":[{"t":"Str","c":"![][" + kv[1] + "]"}]}
    return None

def recurse(x):
    if isinstance(x, list):
        lst = []
        for item in x:
            if isinstance(item, dict) and 't' in item and 'c' in item:
                res = include(item['t'], item['c'])
                if res is None:
                    lst.append(recurse(item))
                elif isinstance(res, list):
                    for z in res: lst.append(z)
                else:
                    lst.append(res)
            else:
                lst.append(recurse(item))
        return lst
    elif isinstance(x, dict):
        dct = {}
        for k in x:
            dct[k] = recurse(x[k])
        return dct
    else:
        return x

if __name__ == "__main__":
    doc = json.loads(sys.stdin.read())
    ret = recurse(doc)
    json.dump(ret, sys.stdout)



import random
import os

script_dir = os.path.dirname(os.path.abspath(__file__))

wiggle = 4

with open(script_dir + "/spheres_triangle.ecs", 'w') as f:
    for y in [-50, -25, 0, 25, 50]:
        for z in [0, 30, 60, 90, 120, 150, 180]:
            for x in [-40, 0, 40, 80, 120]:
                f.write(f"--triangle-sphere {x+random.randint(-wiggle, wiggle)} {y+random.randint(-wiggle, wiggle)} {z+random.randint(-wiggle, wiggle)} {random.randint(5, 13)} 100\n")
    f.write("--vp 8.961043358 58.70862198 -94.01120758 --vi 9.361638069 57.54679489 -91.27469635 --vu 0 1 0 --fov 90 --righthanded\n")

with open(script_dir + "/spheres_quads.ecs", 'w') as f:
    for y in [-50, -25, 0, 25, 50]:
        for z in [0, 30, 60, 90, 120, 150, 180]:
            for x in [-40, 0, 40, 80, 120]:
                f.write(f"--quad-sphere {x+random.randint(-wiggle, wiggle)} {y+random.randint(-wiggle, wiggle)} {z+random.randint(-wiggle, wiggle)} {random.randint(5, 13)} 100\n")
    f.write("--vp 8.961043358 58.70862198 -94.01120758 --vi 9.361638069 57.54679489 -91.27469635 --vu 0 1 0 --fov 90 --righthanded\n")

with open(script_dir + "/spheres_grids.ecs", 'w') as f:
    for y in [-50, -25, 0, 25, 50]:
        for z in [0, 30, 60, 90, 120, 150, 180]:
            for x in [-40, 0, 40, 80, 120]:
                f.write(f"--grid-sphere {x+random.randint(-wiggle, wiggle)} {y+random.randint(-wiggle, wiggle)} {z+random.randint(-wiggle, wiggle)} {random.randint(5, 13)} 100\n")
    f.write("--vp 8.961043358 58.70862198 -94.01120758 --vi 9.361638069 57.54679489 -91.27469635 --vu 0 1 0 --fov 90 --righthanded\n")

with open(script_dir + "/spheres_points.ecs", 'w') as f:
    for y in [-50, -25, 0, 25, 50]:
        for z in [0, 30, 60, 90, 120, 150, 180]:
            for x in [-40, 0, 40, 80, 120]:
                f.write(f"--point-sphere {x+random.randint(-wiggle, wiggle)} {y+random.randint(-wiggle, wiggle)} {z+random.randint(-wiggle, wiggle)} {random.randint(1, 100) / 10} {random.randint(5, 13)} 100\n")
    f.write("--vp 8.961043358 58.70862198 -94.01120758 --vi 9.361638069 57.54679489 -91.27469635 --vu 0 1 0 --fov 90 --righthanded\n")

types = [
    "linear_flat",
    "linear_round",
    "bezier_flat",
    "bezier_round",
    "bezier_normaloriented",
    "bspline_flat",
    "bspline_round",
    "bspline_normaloriented",
    "hermite_flat",
    "hermite_round",
    "hermite_normaloriented",
    "catmulrom_flat",
    "catmulrom_round",
    "catmulrom_normaloriented",
    ]


for t in types:
    id = 1
    with open(script_dir + "/furball_" + t + ".xml", 'w') as f:
        f.write("<?xml version=\"1.0\"?>\n")
        f.write("<scene>")
        for y in [-50, -25, 0, 25, 50]:
            for z in [0, 30, 60, 90, 120, 150, 180]:
                for x in [-40, 0, 40, 80, 120]:
                    f.write(f"  <Group id=\"{id}\">\n")
                    f.write(f"    <Transform id=\"{id+1}\">\n")
                    f.write(f"      <AffineSpace>\n")
                    f.write(f"        1.000000 0.000000 0.000000 {x+random.randint(-wiggle, wiggle)}\n")
                    f.write(f"        0.000000 1.000000 0.000000 {y+random.randint(-wiggle, wiggle)}\n")
                    f.write(f"        0.000000 0.000000 1.000000 {z+random.randint(-wiggle, wiggle)}\n")
                    f.write(f"      </AffineSpace>\n")
                    f.write(f"      <FurBall id=\"{id+2}\" radius=\"{random.randint(3,11)}\" hairtype=\"{t}\">\n")
                    f.write(f"        <material>\n")
                    f.write(f"          <code>\"OBJ\"</code>\n")
                    f.write(f"          <parameters>\n")
                    f.write(f"            <float3 name=\"Kd\">{random.randint(5,100)/100} {random.randint(5,100)/100} {random.randint(5,100)/100}</float3>\n")
                    f.write(f"          </parameters>\n")
                    f.write(f"        </material>\n")
                    f.write(f"      </FurBall>\n")
                    f.write(f"    </Transform>\n")
                    f.write(f"  </Group>\n")
                    id = id + 3

        f.write("</scene>\n")


#!/usr/bin/env python3

## Copyright 2009-2021 Intel Corporation
## SPDX-License-Identifier: Apache-2.0


import sys
import subprocess
import os
import pickle
import re
import shutil

cwd = os.getcwd()

g_debugMode = False
g_intensity = 2

def escape(str):
  str = str.replace("\\",r"\\")
  str = str.replace("\"",r"\"")
  return str

def parse_version(v):
  return tuple(map(int, v.split(".")))

# change some CMake paths in the CMakeCache.txt file for execution of tests
# on potentially different machine (e.g, copied build dir in CI)
def fix_cmake_paths():
  with open('build/CMakeCache.txt', 'r') as file:
      file_content = file.read()

  file_content = re.sub(r"(For build in directory: ).*",        os.path.join(r"\1"+cwd, "build"), file_content)
  file_content = re.sub(r"(embree[0-9]+_BINARY_DIR:STATIC=).*", os.path.join(r"\1"+cwd, "build"), file_content)
  file_content = re.sub(r"(CMAKE_CACHEFILE_DIR:INTERNAL=).*",   os.path.join(r"\1"+cwd, "build"), file_content)

  file_content = re.sub(r"(embree[0-9]+_SOURCE_DIR:STATIC=).*", r"\1"+cwd, file_content)
  file_content = re.sub(r"(CMAKE_HOME_DIRECTORY:INTERNAL=).*",  r"\1"+cwd, file_content)

  with open('build/CMakeCache.txt', 'w') as file:
      file.write(file_content)

# detect platform
if sys.platform.startswith("win"):
  import ctypes
  SEM_FAILCRITICALERRORS = 0x0001
  SEM_NOGPFAULTERRORBOX  = 0x0002
  SEM_NOOPENFILEERRORBOX = 0x8000
  ctypes.windll.kernel32.SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX | SEM_NOOPENFILEERRORBOX);
  OS = "windows"
elif sys.platform.startswith("cygwin"):
  import ctypes
  SEM_FAILCRITICALERRORS = 0x0001
  SEM_NOGPFAULTERRORBOX  = 0x0002
  SEM_NOOPENFILEERRORBOX = 0x8000
  ctypes.cdll.kernel32.SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX | SEM_NOOPENFILEERRORBOX);
  OS = "windows"
elif sys.platform.startswith("linux"):
  OS = "linux"
elif sys.platform.startswith("darwin"):
  OS = "macosx"
else:
  print("unknown platform: "+ sys.platform);
  sys.exit(1)

NAS = os.environ["STORAGE_PATH"] + "/packages/apps"
if "klocwork:ON" in sys.argv:
  NAS = "/NAS/packages/apps"

# configures tests for specified host machine
def runConfig(config):

  conf = []  # CMake configuration
  env = []  # shell environment
  rtcore = [] # rtcore configuration

  build = config["build"]
  conf.append("-D CMAKE_BUILD_TYPE="+build+"")

  cmake_build_suffix = ""
  threads = "0"

  if "threads" in config:
    threads = config["threads"]

  # TODO: not used in any of the workflow definitions
  if "memcheck" in config:
    conf.append("-D EMBREE_TESTING_MEMCHECK="+config["memcheck"]+"")

  # TODO: this is not used anywhere in embree cmake...
  if "sde" in config:
    conf.append("-D EMBREE_TESTING_SDE="+config["sde"]+"")

  if "addrsanitizer" in config:
    conf.append("-D EMBREE_ADDRESS_SANITIZER="+config["addrsanitizer"]+"")

  if "intensity" in config:
    g_intensity = config["intensity"]
  else:
    g_intensity = 2

  if "klocwork" in config:
    conf.append("-D EMBREE_TESTING_KLOCWORK="+config["klocwork"])

  if "L0RTAS" in config:
    conf.append("-D EMBREE_SYCL_L0_RTAS_BUILDER="+config["L0RTAS"])

  if "package" in config:
    conf.append("-D EMBREE_STACK_PROTECTOR=ON")

  if "maxinstancelevelcount" in config:
    conf.append("-D EMBREE_MAX_INSTANCE_LEVEL_COUNT="+config["maxinstancelevelcount"])

  enable_sycl_support = False
  if "EMBREE_SYCL_SUPPORT" in config:
    enable_sycl_support = True
    conf.append("-D EMBREE_SYCL_SUPPORT="+config["EMBREE_SYCL_SUPPORT"])
    conf.append("-D EMBREE_TESTING_ONLY_SYCL_TESTS=ON")

  #if "package" in config and OS == 'linux': # we need up to date cmake for RPMs to work properly
  compiler = config["compiler"]
  platform = config["platform"]
  if OS == "windows":
    cmake_build_suffix = "-- /m /t:rebuild"
    ext = ""
    if platform == "x64":
      ext = " Win64"
    if (compiler == "V142"):
      conf.append("-G \"Visual Studio 16 2019\"")
      conf.append("-T \"V142\"")
      conf.append("-A \"x64\"")
    elif (compiler == "V141"):
      conf.append("-G \"Visual Studio 15 2017"+ext+"\"")
      conf.append("-T \"V141\"")
    elif (compiler == "ICC19-VC141"):
      conf.append("-G \"Visual Studio 15 2017"+ext+"\"")
      conf.append("-T \"Intel C++ Compiler 19.0\"")
    elif (compiler == "ICC18-VC141"):
      conf.append("-G \"Visual Studio 15 2017"+ext+"\"")
      conf.append("-T \"Intel C++ Compiler 18.0\"")
    elif (compiler == "V140"):
      conf.append("-G \"Visual Studio 14 2015"+ext+"\"")
      conf.append("-T \"V140\"")
    elif (compiler == "V120"):
      conf.append("-G \"Visual Studio 12 2013"+ext+"\"")
      conf.append("-T \"V120\"")
    elif (compiler == "V110"):
      conf.append("-G \"Visual Studio 11 2012"+ext+"\"")
      conf.append("-T \"V110\"")
    elif (compiler == "ICC19-VC14"):
      conf.append("-G \"Visual Studio 14 2015"+ext+"\"")
      conf.append("-T \"Intel C++ Compiler 19.0\"")
    elif (compiler == "ICC18-VC14"):
      conf.append("-G \"Visual Studio 14 2015"+ext+"\"")
      conf.append("-T \"Intel C++ Compiler 18.0\"")
    elif (compiler == "ICC17-VC14"):
      conf.append("-G \"Visual Studio 14 2015"+ext+"\"")
      conf.append("-T \"Intel C++ Compiler 17.0\"")
    elif (compiler == "ICC17-VC12"):
      conf.append("-G \"Visual Studio 12 2013"+ext+"\"")
      conf.append("-T \"Intel C++ Compiler 17.0\"")
    elif (compiler == "ICC17"):
      conf.append("-G \"Visual Studio 12 2013"+ext+"\"")
      conf.append("-T \"Intel C++ Compiler 17.0\"")
    elif (compiler == "ICC16"):
      conf.append("-G \"Visual Studio 12 2013"+ext+"\"")
      conf.append("-T \"Intel C++ Compiler 16.0\"")
    elif (compiler == "ICC15"):
      conf.append("-G \"Visual Studio 12 2013"+ext+"\"")
      conf.append("-T \"Intel C++ Compiler XE 15.0\"")
    elif (compiler == "LLVM_CLANG"):
      conf.append("-G \"Visual Studio 16 2019\"")
      conf.append("-A "+platform)
      conf.append("-T \"LLVM_v142\"")
    elif (compiler == "V141_CLANG"):
      conf.append("-G \"Visual Studio 15 2017"+ext+"\"")
      conf.append("-T \"v141_clang_c2\"")
    elif (compiler.startswith("ICX")):
      cmake_build_suffix = ""
      conf.append("-G Ninja -D CMAKE_CXX_COMPILER=icx -DCMAKE_C_COMPILER=icx")
    elif (compiler.startswith("dpcpp")):
      cmake_build_suffix=""
      conf.append("-G Ninja -D CMAKE_CXX_COMPILER=clang++ -DCMAKE_C_COMPILER=clang")
    else:
      raise ValueError('unknown compiler: ' + compiler + '')

  elif OS == "linux":
    if (compiler == "GCC"):
      conf.append("-D CMAKE_CXX_COMPILER=g++ -D CMAKE_C_COMPILER=gcc")
    elif (compiler == "CLANG"):
      conf.append("-D CMAKE_CXX_COMPILER=clang++ -D CMAKE_C_COMPILER=clang")
    elif (compiler.startswith("ICX")):
      conf.append("-G Ninja -DCMAKE_CXX_COMPILER=icpx -DCMAKE_C_COMPILER=icx")
    elif (compiler.startswith("ICC")):
      conf.append("-D CMAKE_CXX_COMPILER="+NAS+"/intel/"+compiler[3:]+"/bin/icpc -D CMAKE_C_COMPILER="+NAS+"/intel/"+compiler[3:]+"/bin/icc")
    elif (compiler.startswith("CLANG")):
      conf.append("-D CMAKE_CXX_COMPILER="+NAS+"/clang/v"+compiler[5:]+"/bin/clang++ -D CMAKE_C_COMPILER="+NAS+"/clang/v"+compiler[5:]+"/bin/clang")
    elif (compiler.startswith("dpcpp")):
      conf.append("-G Ninja -D CMAKE_CXX_COMPILER=clang++ -DCMAKE_C_COMPILER=clang")
    else:
      raise ValueError('unknown compiler: ' + compiler + '')

  else:
    if (compiler == "GCC"):
      conf.append("-D CMAKE_CXX_COMPILER=g++ -D CMAKE_C_COMPILER=gcc")
    elif (compiler == "CLANG"):
      conf.append("-D CMAKE_CXX_COMPILER=clang++ -D CMAKE_C_COMPILER=clang")
    elif (compiler.startswith("ICC")):
      conf.append("-D CMAKE_CXX_COMPILER="+NAS+"/intel/"+compiler[3:]+"-osx/compiler/latest/mac/bin/intel64/icpc -D CMAKE_C_COMPILER="+NAS+"/intel/"+compiler[3:]+"-osx/compiler/latest/mac/bin/intel64/icc")
    elif (compiler.startswith("ICX")):
      conf.append("-D CMAKE_CXX_COMPILER=/opt/intel/oneapi/compiler/"+compiler[3:]+"/mac/bin/intel64/icpc")
      conf.append("-D CMAKE_C_COMPILER=/opt/intel/oneapi/compiler/"+compiler[3:]+"/mac/bin/intel64/icc")
    else:
      raise ValueError('unknown compiler: ' + compiler + '')

  if "ispc" in config:
    conf.append("-D EMBREE_ISPC_SUPPORT=ON")
    ispc_compiler = config["ispc"]
    if ispc_compiler.startswith("ispc"):
  
      ispc_version = ispc_compiler[4:]
  
      if ispc_version != "":
  
        if OS == "windows": bin_folder = "bin\\"
        else              : bin_folder = "bin/"
        if parse_version(ispc_version) < parse_version("1.11.0"): bin_folder = ""
  
        if OS == "linux":
          conf.append("-D EMBREE_ISPC_EXECUTABLE="+NAS + "/ispc/"+ispc_version+"-linux/"+bin_folder+"ispc")
        elif OS == "macosx":
          conf.append("-D EMBREE_ISPC_EXECUTABLE="+NAS + "/ispc/"+ispc_version+"-osx/"+bin_folder+"ispc")
        elif OS == "windows":
          conf.append("-D EMBREE_ISPC_EXECUTABLE="+NAS+"\\ispc\\"+ispc_version+"-windows\\"+bin_folder+"ispc.exe")
        else:
          sys.stderr.write("unknown operating system "+OS)
          sys.exit(1)
    else:
      raise ValueError('unknown ISPC compiler: ' + ispc_compiler + '')
  else:
    conf.append("-D EMBREE_ISPC_SUPPORT=OFF")

  isa = config["isa"]
  if type(isa) == str:
    conf.append("-D EMBREE_MAX_ISA="+isa+"")
  else:
    conf.append("-D EMBREE_MAX_ISA=NONE")
    if "SSE2"      in isa: conf.append("-D EMBREE_ISA_SSE2=ON")
    else                 : conf.append("-D EMBREE_ISA_SSE2=OFF")
    if "SSE42"     in isa: conf.append("-D EMBREE_ISA_SSE42=ON")
    else                 : conf.append("-D EMBREE_ISA_SSE42=OFF")
    if "AVX"       in isa: conf.append("-D EMBREE_ISA_AVX=ON")
    else                 : conf.append("-D EMBREE_ISA_AVX=OFF")
    if "AVX2"      in isa: conf.append("-D EMBREE_ISA_AVX2=ON")
    else                 : conf.append("-D EMBREE_ISA_AVX2=OFF")
    if "AVX512"    in isa: conf.append("-D EMBREE_ISA_AVX512=ON")
    else                 : conf.append("-D EMBREE_ISA_AVX512=OFF")

  if "tasking" in config:
    tasking  = config["tasking"]
    if tasking == "INT":
      conf.append("-D EMBREE_TASKING_SYSTEM=INTERNAL")
    elif tasking == "PPL":
      conf.append("-D EMBREE_TASKING_SYSTEM=PPL")
    elif tasking.startswith("TBB"):
      conf.append("-D EMBREE_TASKING_SYSTEM=TBB")

      if OS == "linux":
        if tasking == "TBB":
          conf.append("-D EMBREE_TBB_ROOT=/usr")
        elif tasking.startswith("TBB"):
          conf.append("-D EMBREE_TBB_ROOT="+NAS+"/tbb/tbb-"+tasking[3:]+"-linux")
        else:
          raise ValueError('unknown tasking system: ' + tasking + '')

      elif OS == "macosx":
        if tasking == "TBB":
          conf.append("-D EMBREE_TBB_ROOT=/opt/local")
        elif tasking == "TBB_HOMEBREW":
          conf.append("-D EMBREE_TBB_ROOT=/opt/homebrew")
        elif tasking.startswith("TBB"):
          conf.append("-D EMBREE_TBB_ROOT="+NAS+"/tbb/tbb-"+tasking[3:]+"-osx")
        else:
          raise ValueError('unknown tasking system: ' + tasking + '')

      elif OS == "windows":
        tbb_path = ""+NAS+"\\tbb\\tbb-"+tasking[3:]+"-windows"
        conf.append("-D EMBREE_TBB_ROOT="+tbb_path)

        # prepend PATH modification to prevent problems with non-delayed
        # evaluation of variables in cmd when running oneAPI DPC++ compiler
        # setup script, for example.
        if platform == "x64":
          env.insert(0, "set PATH="+tbb_path+"\\bin\\intel64\\vc12;"+tbb_path+"\\bin\\intel64\\vc14;"+tbb_path+"\\redist\\intel64\\vc12;"+tbb_path+"\\redist\\intel64\\vc14;%PATH%")
        else:
          env.insert(0, "set PATH="+tbb_path+"\\bin\\ia32\\vc12;"+tbb_path+"\\bin\\ia32\\vc14;"+tbb_path+"\\redist\\ia32\\vc12;"+tbb_path+"\\redist\\ia32\\vc14;%PATH%")

      else:
        sys.stderr.write("unknown operating system "+OS)
        sys.exit(1)

    else:
      raise ValueError('unknown tasking system: ' + tasking)

  if "api_namespace" in config:
    conf.append("-D EMBREE_API_NAMESPACE="+config["api_namespace"])
    conf.append("-D EMBREE_LIBRARY_NAME="+config["api_namespace"])  # we test different library name at the same time
    conf.append("-D EMBREE_ISPC_SUPPORT=OFF")

  # Add embree specific optons
  for opt in ["LEVEL_ZERO", "STATIC_LIB", "TUTORIALS", "BACKFACE_CULLING", "BACKFACE_CULLING_CURVES", "IGNORE_INVALID_RAYS", "FILTER_FUNCTION", "LARGEGRF", "RAY_MASK", "RAY_PACKETS", "STAT_COUNTERS", "COMPACT_POLYS", "MIN_WIDTH"]:
    if opt in config:
      conf.append("-D EMBREE_"+opt+"="+config[opt])

  # TODO: check if we want to chonge the names of theese options, so that the pattern fits here as well
  if "TRI" in config:
    conf.append("-D EMBREE_GEOMETRY_TRIANGLE="+config["TRI"])
  if "QUAD" in config:
    conf.append("-D EMBREE_GEOMETRY_QUAD="+config["QUAD"])
  if "GRID" in config:
    conf.append("-D EMBREE_GEOMETRY_GRID="+config["GRID"])
  if "CURVE" in config:
    conf.append("-D EMBREE_GEOMETRY_CURVE="+config["CURVE"])
  if "SUBDIV" in config:
    conf.append("-D EMBREE_GEOMETRY_SUBDIVISION="+config["SUBDIV"])
  if "USERGEOM" in config:
    conf.append("-D EMBREE_GEOMETRY_USER="+config["USERGEOM"])
  if "INSTANCE" in config:
    conf.append("-D EMBREE_GEOMETRY_INSTANCE="+config["INSTANCE"])
  if "INSTANCE_ARRAY" in config:
    conf.append("-D EMBREE_GEOMETRY_INSTANCE_ARRAY="+config["INSTANCE_ARRAY"])
  if "POINT" in config:
    conf.append("-D EMBREE_GEOMETRY_POINT="+config["POINT"])
  if "GLFW" in config:
    conf.append("-D EMBREE_TUTORIALS_GLFW="+config["GLFW"])
  if "frequency_level" in config:
    rtcore.append("frequency_level="+config["frequency_level"])

  if "sycl" in config:
      conf.append("-D EMBREE_SYCL_AOT_DEVICES="+config["sycl"])
  if "implicit_dispatch_globals" in config:
    conf.append("-D EMBREE_SYCL_IMPLICIT_DISPATCH_GLOBALS="+config["implicit_dispatch_globals"])
  if "sycl_test" in config:
    conf.append("-D EMBREE_SYCL_TEST="+config["sycl_test"])
  if "rt_validation_api" in config:
    conf.append("-D EMBREE_SYCL_RT_VALIDATION_API="+config["rt_validation_api"])
  if "test_only_sycl" in config:
    conf.append("-D EMBREE_TESTING_ONLY_SYCL_TESTS="+config["test_only_sycl"])
  else:
    conf.append("-D EMBREE_TESTING_ONLY_SYCL_TESTS=OFF")

  if "EMBREE_USE_GOOGLE_BENCHMARK" in config:
    conf.append("-D EMBREE_USE_GOOGLE_BENCHMARK="+config["EMBREE_USE_GOOGLE_BENCHMARK"])
  else:
    conf.append("-D EMBREE_USE_GOOGLE_BENCHMARK=OFF")
  if "EMBREE_GOOGLE_BENCHMARK_DIR" in config:
    conf.append("-D benchmark_DIR:PATH="+config["EMBREE_GOOGLE_BENCHMARK_DIR"])

  if "EMBREE_BUILD_GOOGLE_BENCHMARK_FROM_SOURCE" in config:
    conf.append("-D EMBREE_BUILD_GOOGLE_BENCHMARK_FROM_SOURCE=ON")
  else:
    conf.append("-D EMBREE_BUILD_GOOGLE_BENCHMARK_FROM_SOURCE=OFF")

  if "package" in config:
    SIGN_FILE = ""
    if OS == "windows":
      SIGN_FILE = os.environ["SIGN_FILE_WINDOWS"]
    elif OS == "linux":
      SIGN_FILE = os.environ["SIGN_FILE_LINUX"]
    elif OS == "macosx":
      SIGN_FILE = os.environ["SIGN_FILE_MAC"]
    conf.append("-D EMBREE_SIGN_FILE="+SIGN_FILE)

    conf.append("-D BUILD_TESTING=ON")
    conf.append("-D EMBREE_TESTING_INSTALL_TESTS=ON")
    conf.append("-D EMBREE_TESTING_PACKAGE=ON")
    if OS == "linux" and config["package"] == "ZIP":
      conf.append("-D EMBREE_INSTALL_DEPENDENCIES=ON")
      conf.append("-D EMBREE_BUILD_GLFW_FROM_SOURCE=ON")
      conf.append("-D EMBREE_ZIP_MODE=ON")
      conf.append("-D CMAKE_SKIP_INSTALL_RPATH=OFF")
      conf.append("-D CMAKE_INSTALL_INCLUDEDIR=include")
      conf.append("-D CMAKE_INSTALL_LIBDIR=lib")
      conf.append("-D CMAKE_INSTALL_DOCDIR=doc")
      conf.append("-D CMAKE_INSTALL_BINDIR=bin")
      conf.append("-D CMAKE_INSTALL_TESTDIR=testing")
    elif OS == "macosx" and config["package"] == "ZIP":
      conf.append("-D EMBREE_INSTALL_DEPENDENCIES=ON")
      conf.append("-D EMBREE_ZIP_MODE=ON")
      conf.append("-D CMAKE_SKIP_INSTALL_RPATH=OFF")
      conf.append("-D CMAKE_MACOSX_RPATH=ON")
      conf.append("-D CMAKE_INSTALL_INCLUDEDIR=include")
      conf.append("-D CMAKE_INSTALL_LIBDIR=lib")
      conf.append("-D CMAKE_INSTALL_DOCDIR=doc")
      conf.append("-D CMAKE_INSTALL_BINDIR=bin")
      conf.append("-D CMAKE_INSTALL_TESTDIR=testing")
    elif OS == "windows" and config["package"] == "ZIP":
      conf.append("-D EMBREE_INSTALL_DEPENDENCIES=ON")
      conf.append("-D EMBREE_BUILD_GLFW_FROM_SOURCE=ON")
      conf.append("-D EMBREE_ZIP_MODE=ON")
      conf.append("-D CMAKE_INSTALL_INCLUDEDIR=include")
      conf.append("-D CMAKE_INSTALL_LIBDIR=lib")
      conf.append("-D CMAKE_INSTALL_DATAROOTDIR=")
      conf.append("-D CMAKE_INSTALL_DOCDIR=doc")
      conf.append("-D CMAKE_INSTALL_BINDIR=bin")
      conf.append("-D CMAKE_INSTALL_TESTDIR=testing")
    else:
      sys.stderr.write("unknown package mode: "+OS+":"+config["package"])
      sys.exit(1)

  if rtcore:
    conf.append("-D EMBREE_CONFIG="+(",".join(rtcore)))
    
  if "dumptests" in config:
    conf.append("-D EMBREE_TESTING_DUMPTESTS=" + config["dumptests"])
  if "dumpformat" in config:
    conf.append("-D EMBREE_TESTING_DUMPFORMAT=" + config["dumpformat"])

  ctest_suffix = ""
  ctest_suffix += " -D EMBREE_TESTING_INTENSITY="+str(g_intensity)
  if "klocwork" in config:
    ctest_suffix += " -D EMBREE_TESTING_KLOCWORK="+config["klocwork"]

  ctest_suffix += " -D CTEST_CONFIGURATION_TYPE=\""+build+"\""
  ctest_suffix += " -D CTEST_BUILD_OPTIONS=\"" + escape(" ".join(conf))+"\""
  ctest_env = ""
  for e in env:
    ctest_env += e + " && "

  ctest_conf = [ctest_env, ctest_suffix, cmake_build_suffix, threads]
  pickle.dump(ctest_conf, open(".ctest_conf", "wb"), 0)

# builds or runs tests for specified host machine
def run(mode):

  [ctest_env, ctest_suffix, cmake_build_suffix, threads] = pickle.load(open(".ctest_conf", "rb"))

  # pick sde executable
  # don't use sde if no sde cpuid is specified
  sde_cmd = ""
  if mode == "test" and "sde" in g_config:
    if OS == "linux":
      sde_cmd = os.path.sep.join([NAS, 'sde', 'lin', 'sde64'])
    elif OS == "macosx":
      sde_cmd = os.path.sep.join([NAS, 'sde', 'mac', 'sde64'])
    elif OS == "windows":
      sde_cmd = os.path.sep.join([NAS, 'sde', 'win', 'sde64'])
    else:
      sys.stderr.write("unknown operating system "+OS)
      sys.exit(1)
    
    sde_cmd = sde_cmd + " -" + g_config['sde'] + " -- "

  if mode == "test" or mode == "build":
    cmd = ctest_env + sde_cmd + "ctest -VV -S "+ os.path.join("scripts","test.cmake -DSTAGE="+mode+" -DTHREADS="+threads+" -DBUILD_SUFFIX=\""+cmake_build_suffix+"\"") + ctest_suffix
  else:
    cmd = ctest_env + os.path.join("scripts",mode)

  if mode == "env":
    cmd = ctest_env + "echo env";

  if mode == "test" and not OS == "windows":
    fix_cmake_paths()

  # execute step
  if (g_debugMode):
    print(cmd)
  else:
    try:
      if OS == "windows":
        subprocess.check_call(cmd, stderr=subprocess.STDOUT, shell=True)
      else:
        subprocess.check_call(cmd, stderr=subprocess.STDOUT, shell=True, executable='/bin/bash')
    except subprocess.CalledProcessError as e:
      sys.stderr.write("windows test invocation failed with return code "+str(e.returncode))
      sys.exit(1)

g_config = {}
def parseCommandLine(argv):
  global g_config
  global g_debugMode
  if len(argv) == 0:
    return;
  elif len(argv)>=1 and argv[0] == "--debug":
    g_debugMode = True
    parseCommandLine(argv[1:len(argv)])
  elif len(argv)>=1 and argv[0] == "--help":
    return
  elif ':' in argv[0]:
    p = argv[0].split(":")
    if p[0] == "isas":
      g_config["isa"] = p[1].split('-')
    else:
      g_config[p[0]] = p[1]
    parseCommandLine(argv[1:len(argv)])
  else:
    sys.stderr.write("unknown command line option: "+argv[0])
    sys.exit(1)

argv = sys.argv
g_mode = ""
if len(argv) < 2:
  sys.exit(1)
else:
  g_mode = argv[1]
  parseCommandLine(argv[2:len(argv)])

if (g_mode == "configure"):
  runConfig(g_config)
else:
  run(g_mode)



from email.mime import base
import glob
import os
import subprocess
import platform
import time
import sys

script_dir = os.path.dirname(os.path.abspath(__file__))

verbose = False

if len(sys.argv) < 3:
    print("""
Compute reference images for model tests.
needs the embree_viewer or embree_pathtracer respectively in the build folder
usage: 
    python make_reference.py <renderer> <tests_file>
where
    <renderer>    - one of [viewer, pathtracer]
    <tests_file>  - file containing a list of tests as relative paths to .ecs files e.g. tests/models/tests_all.txt

example:
    python tests/models/make_reference.py viewer tests/models/tests_all.txt

     - computes reference images for all tests in tests_all.txt using the viewer OVERWRITING old images in folder 
""")
    exit()

renderer = os.path.sep.join([script_dir, "..", "..", "build", "embree_" + sys.argv[1]])

fin = sys.argv[2]
tests = []
with open(fin, 'r') as f:
    tests = [l for l in [l.strip() for l in f.readlines()] if l and not l.startswith("#")]

if not os.path.exists(script_dir + os.path.sep + "reference"):
    os.mkdir(script_dir + os.path.sep + "reference")

errs = []
tt = time.time()
dirname = os.path.dirname(fin)
for t in tests:
    (path, filename) = os.path.split(t)
    (basename, ext) = os.path.splitext(filename)

    outimage = f"{os.path.sep.join([script_dir, 'reference', sys.argv[1] + '_' + path.replace(os.path.sep, '_')])}_{basename}.exr"
    cmd = f"{renderer} -c {dirname + os.path.sep + t} -o {outimage}"

    if verbose:
        print(cmd)
    if os.system(cmd) == 0:
        print(f"Rendered {outimage}")
    else:
        errs.append(dirname + os.path.sep + t)



    if sys.argv[1] == "viewer":
        outimage = f"{os.path.sep.join([script_dir, 'reference', sys.argv[1] + '_quad_coherent_' + path.replace(os.path.sep, '_')])}_{basename}.exr"
        cmd = f"{renderer} -c {dirname + os.path.sep + t} -o {outimage} -convert-triangles-to-quads --coherent"
        if verbose:
            print(cmd)
        os.system(cmd)
        if os.system(cmd) == 0:
            print(f"Rendered {outimage}")
        else:
            if dirname + os.path.sep + t not in errs:
                errs.append(dirname + os.path.sep + t)

print (time.time() - tt)

if errs:
    print("There have been errors!")
    for e in errs:
        print(f"  - {e}")
    print("Could not be rendered")


#!/usr/bin/env python3

## Copyright 2009-2021 Intel Corporation
## SPDX-License-Identifier: Apache-2.0

import sys
import json
import glob

sys.argv
if len(sys.argv) < 3:
  print("usage: merge_json.py common_file_prefix output_filename")
  sys.exit(1)

common_file_prefix = sys.argv[1]
output_filename = sys.argv[2]

files = glob.glob(common_file_prefix+"*.json")
files.sort()

if (len(files) < 1):
  sys.exit(1)

hero_file = files[0] 
with open(hero_file, 'r') as f:
  hero_data = json.load(f)

for i in range(1, len(files)):
  with open(files[i], 'r') as f:
    data = json.load(f)
    hero_data['benchmarks'].extend(data['benchmarks'])

with open(output_filename, 'w') as f:
  f.write(json.dumps(hero_data, indent=2))


#!/usr/bin/env python3

## Copyright 2009-2021 Intel Corporation
## SPDX-License-Identifier: Apache-2.0

## Used to generate code in kernels/common/motion_derivative.h


import sympy as sp
import numpy as np
import math


################################################################################
#### Utils
################################################################################

def getTerms(map, key):
    if key in map.keys():
        return map[key]
    return 0

# simple linear interpolation wrapper
def lerp(v0,v1,t):
    return v0*(1-t)+v1*t

# custom quaternion to matrix conversion
def to_rotation_matrix(q):
    return sp.Matrix([[q.a*q.a + q.b*q.b - q.c*q.c - q.d*q.d, 2*(q.b*q.c - q.a*q.d),                 2*(q.b*q.d + q.a*q.c),                 0],
                      [2*(q.b*q.c + q.a*q.d),                 q.a*q.a - q.b*q.b + q.c*q.c - q.d*q.d, 2*(q.c*q.d - q.a*q.b),                 0],
                      [2*(q.b*q.d - q.a*q.c),                 2*(q.c*q.d + q.a*q.b),                 q.a*q.a - q.b*q.b - q.c*q.c + q.d*q.d, 0],
                      [0,                                     0,                                     0,                                     1]])


################################################################################
#### Set up symbolic objects
################################################################################

t, theta = sp.symbols("t, theta", real = True)

px0, py0, pz0 = sp.symbols("px0, py0, pz0", real=True) # vertex position at t=0
px1, py1, pz1 = sp.symbols("px1, py1, pz1", real=True) # vertex position at t=1

tx0, ty0, tz0 = sp.symbols("tx0, ty0, tz0", real=True) # translation at t=0
tx1, ty1, tz1 = sp.symbols("tx1, ty1, tz1", real=True) # translation at t=1

qx0, qy0, qz0, qw0 = sp.symbols("qx0, qy0, qz0, qw0", real=True) # quaternion at t=0 
qx1, qy1, qz1, qw1 = sp.symbols("qx1, qy1, qz1, qw1", real=True) # quaternion at t=1

# coefficients for upper triangular matrices
s000, s001, s002, s003, s011, s012, s013, s022, s023 = sp.symbols("s000, s001, s002, s003, s011, s012, s013, s022, s023", real=True)
s100, s101, s102, s103, s111, s112, s113, s122, s123 = sp.symbols("s100, s101, s102, s103, s111, s112, s113, s122, s123", real=True)

q0 = sp.Quaternion(qw0, qx0, qy0, qz0)
q1 = sp.Quaternion(qw1, qx1, qy1, qz1)

# assuming that q1 is qperp = normalize(q1-q0*cosTheta), where cosTheta=dot(q0, q1) and theta = acos(cosTheta).
# this simplifies the terms of the symbolic expressions later
qt = q0 * sp.cos(t*theta) + q1 * sp.sin(t*theta)

S0 = sp.Matrix([[s000, s001, s002, s003],
                [   0, s011, s012, s013],
                [   0,    0, s022, s023],
                [   0,    0,    0,    1]])
S1 = sp.Matrix([[s100, s101, s102, s103],
                [   0, s111, s112, s113],
                [   0,    0, s122, s123],
                [   0,    0,    0,    1]])
D0 = sp.Matrix([[1, 0, 0, tx0],
                [0, 1, 0, ty0],
                [0, 0, 1, tz0],
                [0, 0, 0,  1]])
D1 = sp.Matrix([[1, 0, 0, tx1],
                [0, 1, 0, ty1],
                [0, 0, 1, tz1],
                [0, 0, 0,  1]])
p0 = sp.Matrix([px0, py0, pz0, 1])
p1 = sp.Matrix([px1, py1, pz1, 1])

Gamma = lerp(D0, D1, t)*to_rotation_matrix(qt)*lerp(S0, S1, t)*lerp(p0, p1, t)
C = sp.Matrix(np.empty(8))   # 8 coefficients
K = sp.Matrix(np.empty(7))   # 7 inputs
A = sp.Matrix(np.empty(8*7*3)) # 8 coefficients, 7 inputs (1, px0, py0, pz0, px1, py1, pz1), 3 dimensions (x, y, z)
dGamma = sp.diff(Gamma, t)


################################################################################
#### Group the coefficients (this might time a couple of seconds)
################################################################################

# loop over dimensions (x, y, z)
for dim in range(3):
    dm = sp.expand(dGamma[dim])
    dm = dm.subs(sp.sin(t*theta)*sp.sin(t*theta),(1-sp.cos(2*t*theta))/2) # remove sin(t*theta)^2
    dm = dm.subs(sp.cos(t*theta)*sp.cos(t*theta),(1+sp.cos(2*t*theta))/2) # remove cos(t*theta)^2
    dm = dm.subs(sp.sin(t*theta)*sp.cos(t*theta),sp.sin(2*t*theta)/2)     # remove sin(t*theta)*cos(t*theta)
    dm = sp.expand(dm)

    # group all terms in the form a + b * cos(2*t*theta) + c * sin(2*t*theta)
    dm_cos_sin = sp.collect(dm, (sp.cos(2*t*theta), sp.sin(2*t*theta)), evaluate=False)

    # get the terms
    coeff_cos   = getTerms(dm_cos_sin, sp.cos(2*t*theta))
    coeff_sin   = getTerms(dm_cos_sin, sp.sin(2*t*theta))
    coeff_const = getTerms(dm_cos_sin, 1)

    # group the term in the form a + b * t 
    coeff_const_t = sp.collect(coeff_const, t, evaluate=False)
    C[0] = getTerms(coeff_const_t, 1)
    C[1] = getTerms(coeff_const_t, t)

    # group the term in the form a + b * t + c * t^2 
    coeff_cos_t = sp.collect(coeff_cos, t, evaluate=False)
    C[2] = getTerms(coeff_cos_t, 1)
    C[3] = getTerms(coeff_cos_t, t)
    C[4] = getTerms(coeff_cos_t, t*t)

    # group the term in the form a + b * t + c * t^2 
    coeff_sin_t = sp.collect(coeff_sin, t, evaluate=False)
    C[5] = getTerms(coeff_sin_t, 1)
    C[6] = getTerms(coeff_sin_t, t)
    C[7] = getTerms(coeff_sin_t, t*t)

    for c in range(8):
        kc = sp.collect(C[c], (px0, py0, pz0, px1, py1, pz1), evaluate=False)
        K[0] = getTerms(kc, 1)
        K[1] = getTerms(kc, px0)
        K[2] = getTerms(kc, py0)
        K[3] = getTerms(kc, pz0)
        K[4] = getTerms(kc, px1)
        K[5] = getTerms(kc, py1)
        K[6] = getTerms(kc, pz1)

        for k in range(7):
            K[k] = sp.expand(K[k])
            K[k] = K[k].subs(qw0*qw0, 1-qx0*qx0-qy0*qy0-qz0*qz0) # clean up substitutions
            K[k] = K[k].subs(qw1*qw1, 1-qx1*qx1-qy1*qy1-qz1*qz1) # clean up substitutions
            K[k] = sp.simplify(K[k])
            A[8*7*dim + c*7 + k] = K[k]


################################################################################
#### Write code to file
################################################################################

from sympy.utilities.codegen import codegen, default_datatypes
from sympy.codegen.ast import real, float32
from sympy.printing.ccode import C99CodePrinter
printer = C99CodePrinter()

# custom code printer that will not generate such nonesene as x^2 -> pow(x, 2)
class CustomCodePrinter(C99CodePrinter):
    def _print_Pow(self, expr):
        if expr.exp.is_integer and expr.exp > 0 and expr.exp < 5:
            return '*'.join([self._print(expr.base) for i in range(expr.exp)])
        else:
            return super()._print_Pow(expr)

customprinter = CustomCodePrinter()
customprinter.type_aliases[real] = float32 # cosf instead of cos
default_datatypes["float"].cname = "float" # float instead of double
params = [
    theta,
    tx0, ty0, tz0,
    tx1, ty1, tz1,
    qw0, qx0, qy0, qz0,
    qw1, qx1, qy1, qz1,
    s000, s001, s002, s003, s011, s012, s013, s022, s023,
    s100, s101, s102, s103, s111, s112, s113, s122, s123]
R = sp.MatrixSymbol("coeff", A.shape[0], A.shape[1])
P = sp.MatrixSymbol('p', len(params), 1)
param_map = dict(zip(params, P))
B = A.xreplace(param_map)
codegen(('motion_derivative_coefficients', sp.Eq(R,B)), language='c', printer=customprinter, prefix='motion_derivative_coefficients', to_files=True)
